
# PToolbox package
# Function: useful general perl routines
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 12-17-2004 
#	add the motif level accuracy and sequence/training set level accuracy
#	
# Last modified: 10-25-2004
package PToolbox;
use strict;
use Descriptive;

#******************************************************************************
#FUNCTION: remove duplicate consecutive symbols to squeezeString
#USAGE:    $string="AAAABBB CCC"
#          squeezeString($string, "A B") --> "AB CCC"
sub squeezeString{
	my ($inString,$separator)=@_;
	my @items=split(//,$inString);
	my $curSymbol=substr($inString,0,1);
	my $outString=$curSymbol;
	for(my $i=1;$i<scalar @items;$i++){
		my $matchSeperator=0;
		if($separator eq "" or $separator=~/$curSymbol/){
		   $matchSeperator=1;
		}
		
		if($items[$i] eq $curSymbol and $matchSeperator==1){
			next;
		}
		else{
			$outString=$outString.$items[$i];
			$curSymbol = $items[$i];
		}	
	}
# 	print $outString,"-->out\n";
	return $outString;
}

#******************************************************************************
#FUNCTION: remove surrounding whitespace at the head and tail of a string
#USAGE:    $string="   AAAABBB CCC  "
#          cleanString($string) --> "AAAABBB CCC"
sub cleanString{
	my ($inString) =@_;
	#print "inString=$inString,\n";
	$inString=~/\s*(\S*.*\S)(\s*)/g;
	my $outString = $1;
	#print "===$1 --$2", $outString, "\n";
	return $outString;
}

#******************************************************************************
#FUNCTION: find locations of a regular expression matches
#Return an array of positions where the regular expression
#  appears in the string
#USAGE:    
#          
sub getMatchPositions{
    my($regexp, $sequence) = @_;
    my @positions = (  );
    # Determine positions of regular expression matches
    while ( $sequence =~ /$regexp/ig ) {
        push ( @positions, pos($sequence) - length($&) + 1);
    }
    return @positions;
}

#******************************************************************************
# function: return the maximum and minimum value in an array
# return (min, max)
sub min_and_max{
    my (@numbers);
    @numbers = @_;
    my ($min, $max);
    $min = $numbers[0];
    $max = $numbers[0];

    foreach my $i (@numbers)
    {
        if ($i > $max)
        {
            $max = $i;
        }
        elsif ($i < $min)
        {
            $min = $i;
        }
    }

    return ($min, $max);
}
	
#FUNCTION, print an array in horizonal line seperated by--
sub printH{
	my @data = @_;
	for(my $i=0;$i<scalar @data;$i++){
		if($i !=scalar @data -1){
			print $data[$i], "-";
		}
		else{
			print $data[$i], "\n";
		}
	}
}

#FUNCTION, print an array in horizonal line seperated by--
sub printV{
	my @data = @_;
	for(my $i=0;$i<scalar @data;$i++){
		if($i !=scalar @data -1){
			print "$data[$i]\n";
		}
		else{
			print "$data[$i]\n";
		}
	}
}

#FUNCTION: calculate sum of array elements in a range
sub sumArray{
   my ($start,$length,@data)=@_;
   my $end=0;
   if($length == 0){ #default mode
	$length = scalar @data;
   }
   $end = $start+$length-1;
   my $sum=0;
   for(my $i=$start;$i<=$end;$i++){
   	$sum+=$data[$i];
   }
   return $sum;
}

#get overlap length of two segments
sub getoverlap{
	my ($left1, $right1, $left2, $right2) = @_;
	
	my $oLeft=0;
	my $oRight = 0;
	if($left1>$left2){
		$oLeft = $left1;
	}
	else{
		$oLeft = $left2;
	}
	
	if($right1<$right2){
		$oRight = $right1;
	}
	else{
		$oRight = $right2;
	}
	if($oLeft>$oRight){
		return 0;
	}
	else{
		return $oRight-$oLeft+1;
	}
}

sub getMotifScore{
	my ($motifRecord)= @_;
	my @items = split(':', $motifRecord);
	return $items[1];
}


1;

