#!/usr/bin/perl
#batchMotifP.pl
#function: batch run a motif algorithm to all the sequence files in 
#          folder /step5regulon using the PBS cluster. It will intelligent find
#			the uninitiated experiment No. k by checking
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: April-6-2005


use strict;
use warnings;
use PMotif;


my $rawCommandline = join(" ", @ARGV);

#AlignACE -i folder/file -expect 10 -gcback 0.5 -numcols $expectMotifWidth -odstFolder/ofile
#AlignACE doesn't have -o option, transform to >.  


#-----------------------------------------------------------------------
#	commandline option processing
use Getopt::Std;
my %opts = ();
getopts('c:a:r:f:h:i', \%opts);
$0=~/.*\/(.*)/;
# ':'after symbol means it takes arguments, otherwise just a flag which is 1 when set
sub Usage {
print <<"EOF";

Usage: $1 [-h] [-r<run No.>] [-a <algorithm>]

Examples:
  	$1 -aAlignACE -r0
Description:
	run the motif algorithm on the input sequences in step5regulon
	
	-a algorithm	[AlignACE]
	-c commandline	[""]
	-r the rTH run	[0]
     ...
EOF
    exit;
}
&Usage() if defined $opts{'h'};
my $curAlgorithm= $opts{'a'}||"AlignACE";
my $runNo= $opts{'r'}||0;
my $folder = $opts{'f'}||'step5regulon';
my $infolder = $opts{'i'}||'nowhere';

if($curAlgorithm ne "MDScan" && $curAlgorithm ne "BioProspector"){
	$rawCommandline=~s/-o/>/g;
}

#print $rawCommandline, "**\n";
$rawCommandline=~/.*-c(.*)/;
my $commandline = $1;
#emdrunP.pl -r 1 -a BioProspector -f step5regulon -c BioProspector -i folder/file -w 15 -f ecoli.BioProspector3.bg -d 1 -a 1 -r 5 -odstFolder/ofile


my $dstFolder = "step6$curAlgorithm";
my $expDir="";
if(!(-e $dstFolder)){#first run
    mkdir $dstFolder;
	$expDir ="$dstFolder/exp-0";
	mkdir $expDir;
	`cp -r ./step5regulon $expDir`;
	`cp -r ./*.bg $expDir`;
	#`h -c "cp -r ./knownsites.txt $expDir"`;
}
else{
	opendir(EXPFOLDER, $dstFolder);
	my @dir = grep (!/^\.\.?$/, readdir(EXPFOLDER));
	close(EXPFOLDER);
	@dir= grep (/exp-.*/,@dir);
	my %expNo=();
	foreach (@dir){
		$_=~/exp-(.*)/;
		$expNo{$1}++;
	}
	#find the first lowest unavailable exp No.
	my $expID=0;
	while(1){
		if($expNo{$expID}){
			$expID++;
		}
		else{
			last;
		}
	}
	
	$expID = $runNo; #use specified experiement run no.
	$expDir ="$dstFolder/exp-$expID";
	mkdir $expDir;
	`bash -c "cp -r ./step5regulon $expDir"`;
	`bash -c "cp -r ./*.bg $expDir"`;
	#`bash -c "cp -r ./knownsites.txt $expDir"`;
}
$folder="$expDir/step5regulon";
$dstFolder = "$expDir/$dstFolder";
if(!(-e $dstFolder)){#first run
    mkdir $dstFolder;
}

unless(opendir(FOLDER, "$folder")) {
    print "Cannot open folder $folder!\n";
    exit;
}
# Read the folder, ignoring special entries "." and ".."
my @files = grep (!/^\.\.?$/, readdir(FOLDER));
closedir(FOLDER);

foreach my $file(@files){
	my $command = $commandline;
	my $ofile = $file;
    #print "$file, $ofile, $folder, $dstFolder\n";

	$command=~s/ofile/$ofile/g;
	$command=~s/\/file/\/$file/g;#using \/ to avoid -bfile replacement error of MEME
	$command=~s/folder/$folder/g;
	$command=~s/dstFolder/$dstFolder/g;
	print "$command\n";
	`$command`;
}




