#!/usr/bin/perl
#emd-runP.pl
#function: batch run standalone algorithms for the ensemble motif algorithms
#          to all the sequence files in folder /step5regulon using PBS cluster
#
# Author: Jianjun Hu (hujianju@purdue.edu)
# Kihara Laboratory of Bioinformatics
# Department of Biological Sciences
# Purdue University, West Lafayette, IN, 47906.
#
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# Last modified: Feb-3-2005


use strict;
use warnings;
use PMotif;
use File::Copy;
#-----------------------------------------------------------------------
#	commandline option processing
use Getopt::Std;
my %opts = ();
getopts('w:n:d:f:j:h', \%opts);
$0=~/.*\/(.*)/;

# ':'after symbol means it takes arguments, otherwise just a flag which is 1 when set
sub Usage {
print <<"EOF";

Usage: $1 [-h] [-f< configureFile >] [-w<motifwidth>] [-n <numer of motifs>]
          [-d<target folder of training data]
Examples:
  	$1 -f -n5 -w15 (with default parameters)
Description:
	run the stand-alone algorithms multiple times for ensemble algorithm
	-f configureFile	[emd.cfg]
	-w motifwidth expected	[15]
	-n numer of motifs to find	[5]
	-d taret folder of training data [./step5regulon]
     ...
EOF
    exit;
}
&Usage() if defined $opts{'h'};
my $configFile= $opts{'f'}||"emd.cfg";
my $nMotifs=$opts{'n'}||5;
my $folder = $opts{'d'}||'step5regulon';
my $expectMotifWidth= $opts{'w'}||15;
my $jobTemplate = $opts{'j'}||'runmotif.sh';

if(!(-e $jobTemplate)){
	print "Error: qsub Job file template $jobTemplate doesn't exist!\n";
	exit;
}


my @files = (  );
unless(opendir(FOLDER, $folder)) {
    print "Cannot open folder $folder!\n";
    exit;
}
# Read the folder, ignoring special entries "." and ".."
@files = grep (!/^\.\.?$/, readdir(FOLDER));
# print "Total ",scalar @files, " input files\n";tFold

my $curAlgorithm="AlignACE";
my $dstFolder = "step6$curAlgorithm";
if(!(-e $dstFolder)){
    mkdir $dstFolder;
}

#-----------------------------------------------------------------------
#print "Config file: ", $configFile,"\n";
#read configuration file and run the standalone programs
open(FILE,$configFile);
my @configData=<FILE>;
@configData=grep !/#.*\n$/,@configData;
my $dataline= join("",@configData);
my @groupData = split("Alg:",$dataline);
my $nGroup=scalar @groupData;

#section 1:
chop $groupData[0];
my @items = split(":",$groupData[0]);
my $nAlgorithm = $items[1];

#section 2:
for(my $k=1;$k<=$nAlgorithm;$k++){
# 	print "Running the $k th algorithm\n";
	chop $groupData[$k];
	my @lines = split("\n",$groupData[$k]);
	my ($algorithm, $nRun, $nConfig) = split(":",$lines[0]);
	
	for (my $i=0;$i<$nRun;$i++){
		#randomly pick up a configuration parameter and run it
 		my $whichParamSet = rand($nConfig-1)+1;
		my $commandline=$lines[$whichParamSet];
		$commandline=~s/\$algorithm/$algorithm/g;
		$commandline=~s/\$expectMotifWidth/$expectMotifWidth/g;
		$commandline=~s/\$nMotifs/$nMotifs/g;
		#print $commandline,"\n";
		
		#prepare the job file
		$jobTemplate =~/(.*).sh/;
		my $jobFile = "$1-$algorithm-$i.sh";
		copy($jobTemplate, $jobFile);
		my $shortAlgName= substr($algorithm,0,12);
		
		#update the default jobname.sh to jobnameK.sh
		open(FILE, $jobFile);
		my @lines=<FILE>;
		close(FILE);
		for(my $j=0;$j<scalar @lines;$j++){
			if($lines[$j]=~/#PBS -N (.*)/){
				$lines[$j]="#PBS -N $1$shortAlgName$i\n";
			}
			elsif($lines[$j]=~/#PBS -e (.*).err/){
				$lines[$j]="#PBS -e $1$shortAlgName-$i.err\n";
			}
			elsif($lines[$j]=~/#PBS -o (.*).log/){
				$lines[$j]="#PBS -e $1$shortAlgName-$i.log\n";
			}
			elsif($lines[$j]=~/batchMotifP.pl.*/){
				$lines[$j]="emdrunP.pl -r $i -a $algorithm -f $folder -c $commandline";
			}
		}
		open(OFILE, ">$jobFile");
		print OFILE @lines;
		close(OFILE);
		`qsub $jobFile`;
		}
}



#emd.cfg example
#Alg:MotifSampler:10:10
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile
#$algorithm -f $folder/$file -b ecoli.intergenic.MotifSampler3.bg -s 0 -n $nMotifs -w $expectMotifWidth -r 5 -o$dstFolder/$ofile











